// This class stores elements of type E in nodes. Each node has a link to the next node (if exists).
public class LinkedList<E> implements List<E> {

    private static class Node<E> {
        Node(E e, Node next) {
            this.data = e;
            this.next = next;
        }

        E data;
        Node<E> next;
    }

    @Override
    public int size() {
        int size = 0;
        Node cur = rootNode;
        while (cur != null) {
            size++;
            cur = cur.next;
        }
        return size;
    }

    // is this list empty?
    public boolean isEmpty() {
        return rootNode == null;
    }

    // does the list contain the specified element
    @Override
    public boolean contains(Object element) {
        if (isEmpty()) return false;
        Node n = rootNode;
        while (n != null) {
            if (n.data.equals(element)) return true;
            n = n.next;
        }
        return false;
    }

    // add the passed element to the lists end
    @Override
    public void add(E element) {
        if (rootNode == null) {
            rootNode = new Node<>(element, null);
            return;
        }
        Node cur = rootNode;
        while (cur.next != null)
            cur = cur.next;
        cur.next = new Node(element, null);
    }

    // get the element stored at the provided index
    @Override
    public E get(int index) {
        Node<E> cur = rootNode;
        while (index-- > 0 && cur.next != null)
            cur = cur.next;
        return cur.data;
    }

    // delete the element if it is found in the list, return if it was found and deleted
    @Override
    public boolean delete(E element) {
        if (rootNode != null && rootNode.data.equals(element)) {
            rootNode = rootNode.next;
            return true;
        }
        Node previous = rootNode;
        Node n = rootNode.next;
        while (n != null) {
            if (n.data.equals(element)) {
                previous.next = n.next;
                return true;
            }
            previous = n;
            n = n.next;
        }
        return false;
    }

    // clear deletes all items in list
    @Override
    public void clear() {
        rootNode = null;
    }

    // this is the root node of the list
    private Node<E> rootNode;
}
